/*;  grid.cc {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.5 $$ {\rhead{\sl grid.cc\quad\thepage}} */
/*;  $$Date: 1996/12/09 03:00:41 $$  */
/*;  $$Source: /home/shiiba/96prog/matrix/grid.cc,v $$  */

#ifndef grid_cc
#define grid_cc

#include "grid.h"

template <class D>
Grid<D>::Grid()
{
  R = 1;
  C = 1;
  N = 1;
  
  MmData = new D [N];
  
  if (MmData == NULL)
    {
      error(); abort();
    }
}

template <class D>
Grid<D>::Grid
(
 size_t rows,
 size_t cols
 )
{
  R = rows;
  C = cols;
  N = rows * cols;
  
  if ((R == 0) || (C == 0))
    {
      error(); abort();
    }
    
  if ((N < R) || (N < C))
    {
      error(); abort();
    }
    
  MmData = new D [N];
  
  if (MmData == NULL)
    {
      error(); abort();
    }
}

template <class D>
Grid<D>::Grid
(
 const Grid<D> & m
 )
{
  R = m.R;
  C = m.C;
  N = m.N;
  
  MmData = new D [N];
  
  if (MmData == NULL)
    {
      error(); abort();
    }
    
  for (size_t i = 0; i < N; ++i)
    MmData[i] = m.MmData[i];
}

// destructors
template <class D>
Grid<D>::~Grid()
{
  delete [] MmData;
}

// assignment operator
template <class D>
void Grid<D>::operator =
(
 const Grid<D> & m
 )
{
  R = m.R;
  C = m.C;
  N = m.N;
  
  delete [] MmData;
  
  MmData = new D [N];
  
  if (MmData == NULL)
    {
      error(); abort();
    }
    
  for (size_t i = 0; i < N; ++i)
    MmData[i] = m.MmData[i];
}

template <class D>
void Grid<D>::operator =
(
 const D * array
 )
{
  // note: must assume that array has N members
  const D * aptr = array;
  D * mptr = MmData;
  
  for (size_t i = 0; i < N; ++i)
    {
      (*mptr) = (*aptr);
      ++aptr;
      ++mptr;
    }
}

template <class D>
D Grid<D>::Get
(
 size_t row,
 size_t col
 ) const
{
#ifndef NO_SUBCHK
  if (row > R || col > C)
    {
      cerr << "Invalid subscript. " << endl;
      error();  abort();
    }
#endif
  return MmData[Index(row,col)];
}

template <class D>
D & Grid<D>::operator ()(
  size_t row,
  size_t col
  )
{
#ifndef NO_SUBCHK
  if (row > R || col > C)
    {
      cerr << "Invalid subscript. " << endl;
      error();  abort();
    }
#endif
  return MmData[Index(row,col)];
}

// apply a function to each element
template <class D>
void Grid<D>::Apply
(
 D (* func)(const D & n)
 )
{
  for (size_t i = 0; i < N; ++i)
    MmData[i] = func(MmData[i]);
}

template <class D>
Grid<D> Apply(
  const Grid<D> & m,
  D (* func)(const D & n)
  )
{
  Grid<D> result(m);
  
  for (size_t i = 0; i < m.N; ++i)
    result.MmData[i] = func(result.MmData[i]);
  
  return result;
}

// fill matrix with specific value
template <class D>
void Grid<D>::Fill(
  const D & x
  )
{
  for (size_t i = 0; i < N; ++i)
    MmData[i] = x;
}

// create a row matrix
template <class D>
Grid<D> Grid<D>::VectorRow(
  size_t row
  )
{
  if (row >= R)
    {
      error(); abort();
    }
  
  Grid<D> vector(C,1);
  
  for (size_t i = 0; i < C; ++i)
    vector(i,0) = MmData[Index(row,i)];
  
  return vector;
}

// create a column vector
template <class D>
Grid<D> Grid<D>::VectorCol(
  size_t col
  )
{
  if (col >= C)
    {
      error(); abort();
    }
  
  Grid<D> vector(1,R);
  
  for (size_t i = 0; i < R; ++i)
    vector(0,i) = MmData[Index(i,col)];
  
  return vector;
}

// transpose a matrix
template <class D>
Grid<D> Grid<D>::Transpose()
{
  Grid<D> result(C,R);
  
  const D * tptr = MmData;
  D * rptr;
  
  for (size_t i = 0; i < R; ++i)
    {
      rptr = result.MmData + i;
      
      for (size_t j = 0; j < C; ++j)
	{
	  (*rptr) = (*tptr);
	  ++tptr;
	  rptr += R;
	}
    }
  
  return result;
}

template <class D>
void Grid<D>::Resize(
  size_t rows,
  size_t cols
  )
{
  delete [] MmData;
  
  R = rows;
  C = cols;
  N = rows * cols;
  
  if ((R == 0) || (C == 0))
    {
      error(); abort();
    }
  
  if ((N < R) || (N < C))
    {
      error(); abort();
    }
  
  MmData = new D [N];
  
  if (MmData == NULL)
    {
      error(); abort();
    }
}

#endif

/*; end of file */
